unit WalkUnit;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, FileCtrl, ComCtrls, IniFiles, JPEG, GIFImage, Mask,
  Spin, ImageFuncs, HTMLTemplate;

type
  TMainForm = class(TForm)
    Laufwerk: TDriveComboBox;
    Verzeichnis: TDirectoryListBox;
    Dateien: TFileListBox;
    Progress: TProgressBar;
    Status: TStatusBar;
    OptionsGroup: TGroupBox;
    Start: TButton;
    PreviewImage: TImage;
    About: TButton;
    ExitPro: TButton;
    HTMLFile: TEdit;
    Label1: TLabel;
    Label2: TLabel;
    TemplateSelect: TOpenDialog;
    Template: TEdit;
    ThumbGroup: TGroupBox;
    MakeThumbs: TCheckBox;
    BMPGroup: TGroupBox;
    DeleteBMP: TCheckBox;
    ConvertBMP: TCheckBox;
    UseBMPs: TCheckBox;
    ThumbDir: TEdit;
    ThumbPrefix: TEdit;
    Label3: TLabel;
    Label4: TLabel;
    MaxThumbSize: TMaskEdit;
    Label5: TLabel;
    Spalten: TSpinEdit;
    Label6: TLabel;
    Label7: TLabel;
    Zeilen: TSpinEdit;
    Label8: TLabel;
    PicsPerPage: TLabel;
    Label9: TLabel;
    Label10: TLabel;
    Label11: TLabel;
    ThumbsPresent: TCheckBox;
    procedure ConvertBMPClick(Sender: TObject);
    procedure UseBMPsClick(Sender: TObject);
    procedure ExitProClick(Sender: TObject);
    procedure AboutClick(Sender: TObject);
    procedure TemplateEnter(Sender: TObject);
    procedure CheckBoxes;
    procedure MakeThumbsClick(Sender: TObject);
    procedure SpaltenChange(Sender: TObject);
    procedure ZeilenChange(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure LoadSettings(Profile:string);
    procedure SaveSettings(Profile:string);
    procedure StartClick(Sender: TObject);
    procedure ThumbsPresentClick(Sender: TObject);
  private
    { Private-Deklarationen }
  public
    { Public-Deklarationen }
  end;

Const
  ProgramVersion='1.1';
  ProgramYear='2001';
  ProgramAuthor='Tobias Gruetzmacher';
  ProgramAuthorMail='tobias@portfolio16.de';
  ProgramAuthorHomepage='http://www.portfolio16.de/';
  ProgramComments='Die erste Version dieses Programms wurde 1999 von '+
    'Andreas Golgath fr die PC Professionell entwickelt.'#13#10+
    'Software und Quelltext drfen frei verwendet werden.'#13#10+
    #13#10'This software is based, in part, on the work of Anders Melander.';

var
  MainForm: TMainForm;

implementation

uses AboutUnit;

{$R *.DFM}

Const
  BmpMask=';*.bmp';

var
  TemplateName:string;

procedure TMainForm.ConvertBMPClick(Sender: TObject);
begin
  CheckBoxes;
end;

procedure TMainForm.UseBMPsClick(Sender: TObject);
begin
  CheckBoxes;
  if UseBMPs.Checked then
    Dateien.Mask:=Dateien.Mask+';*.bmp'
  else
    Dateien.Mask:=Copy(Dateien.Mask,1,Pos(BmpMask,Dateien.Mask)-1)+
      Copy(Dateien.Mask,Pos(BmpMask,Dateien.Mask)+Length(BmpMask),Length(Dateien.Mask));
end;

procedure TMainForm.ExitProClick(Sender: TObject);
begin
  if MessageDlg('Wollen Sie Image Walker wirklich beenden?',
      mtConfirmation,[mbYes,mbNo],0)=mrYes then
    Application.Terminate;
end;

procedure TMainForm.AboutClick(Sender: TObject);
begin
  AboutBox.ShowModal;
end;

procedure TMainForm.LoadSettings(Profile:string);
var
  IniDatei:TIniFile;
begin
  IniDatei:=TIniFile.Create(ChangeFileExt(Application.ExeName,'.ini'));
  try
    UseBMPs.Checked:=IniDatei.ReadBool(Profile,'UseBMPs',true);
    ConvertBMP.Checked:=IniDatei.ReadBool(Profile,'ConvertBMP',true);
    DeleteBMP.Checked:=IniDatei.ReadBool(Profile,'DeleteBMP',false);
    MakeThumbs.Checked:=IniDatei.ReadBool(Profile,'MakeThumbs',true);
    ThumbsPresent.Checked:=IniDatei.ReadBool(Profile,'ThumbsPresent',false);
    ThumbDir.Text:=IniDatei.ReadString(Profile,'ThumbDir','thumbs');
    ThumbPrefix.Text:=IniDatei.ReadString(Profile,'ThumbPrefix','d_');
    MaxThumbSize.Text:=IntToStr(IniDatei.ReadInteger(Profile,'MaxThumbSize',300));
    HTMLFile.Text:=IniDatei.ReadString(Profile,'HTMLFile','index.htm');
    TemplateName:=IniDatei.ReadString(Profile,'Template','Vorlage.htm');
    Template.Text:=ExtractFileName(TemplateName);
    Spalten.Value:=IniDatei.ReadInteger(Profile,'Cols',2);
    Zeilen.Value:=IniDatei.ReadInteger(Profile,'Rows',10);
    PicsPerPage.Caption:=IntToStr(Spalten.Value*Zeilen.Value);
    Verzeichnis.Directory:=IniDatei.ReadString(Profile,'LastDir','');
  finally
    IniDatei.Free;
  end;
  CheckBoxes;
end;

procedure TMainForm.SaveSettings(Profile:string);
var
  IniDatei:TIniFile;
begin
  IniDatei:=TIniFile.Create(ChangeFileExt(Application.ExeName,'.ini'));
  try
    IniDatei.WriteBool(Profile,'UseBMPs',UseBMPs.Checked);
    IniDatei.WriteBool(Profile,'ConvertBMP',ConvertBMP.Checked);
    IniDatei.WriteBool(Profile,'DeleteBMP',DeleteBMP.Checked);
    IniDatei.WriteBool(Profile,'MakeThumbs',MakeThumbs.Checked);
    IniDatei.WriteBool(Profile,'ThumbsPresent',ThumbsPresent.Checked);
    IniDatei.WriteString(Profile,'ThumbDir',ThumbDir.Text);
    IniDatei.WriteString(Profile,'ThumbPrefix',ThumbPrefix.Text);
    IniDatei.WriteInteger(Profile,'MaxThumbSize',StrToInt(MaxThumbSize.Text));
    IniDatei.WriteString(Profile,'HTMLFile',HTMLFile.Text);
    IniDatei.WriteString(Profile,'Template',TemplateName);
    IniDatei.WriteInteger(Profile,'Cols',Spalten.Value);
    IniDatei.WriteInteger(Profile,'Rows',Zeilen.Value);
    IniDatei.WriteString(Profile,'LastDir',Verzeichnis.Directory);
  finally
    IniDatei.Free;
  end;
end;

procedure TMainForm.TemplateEnter(Sender: TObject);
begin
  if TemplateSelect.Execute then
    begin
    TemplateName:=TemplateSelect.FileName;
    Template.Text:=ExtractFileName(TemplateName);
    end;
end;

procedure TMainForm.CheckBoxes;
begin
  if UseBMPs.Checked then
    begin
    ConvertBMP.Enabled:=true;
    if ConvertBMP.Checked then
      DeleteBMP.Enabled:=true
    else
      DeleteBMP.Enabled:=false;
    end
  else
    begin
    ConvertBMP.Enabled:=false;
    DeleteBMP.Enabled:=false;
    end;

  if MakeThumbs.Checked or ThumbsPresent.Checked then
    begin
    Label3.Enabled:=true;
    Label4.Enabled:=true;
    ThumbDir.Enabled:=true;
    ThumbPrefix.Enabled:=true;
    end
  else
    begin
    Label3.Enabled:=false;
    Label4.Enabled:=false;
    ThumbDir.Enabled:=false;
    ThumbPrefix.Enabled:=false;
    end;

  if MakeThumbs.Checked then
    begin
    MaxThumbSize.Enabled:=true;
    Label5.Enabled:=true;
    Label11.Enabled:=true;
    end
  else
    begin
    MaxThumbSize.Enabled:=false;
    Label5.Enabled:=false;
    Label11.Enabled:=false;
    end;

  // Only for security, if somebody mixes up the config file
  if MakeThumbs.Checked and ThumbsPresent.Checked then
    ThumbsPresent.Checked:=false;
end;

procedure TMainForm.MakeThumbsClick(Sender: TObject);
begin
  if MakeThumbs.Checked then
    ThumbsPresent.Checked:=false;
  CheckBoxes;
end;

procedure TMainForm.SpaltenChange(Sender: TObject);
begin
  PicsPerPage.Caption:=IntToStr(Spalten.Value*Zeilen.Value);
end;

procedure TMainForm.ZeilenChange(Sender: TObject);
begin
  PicsPerPage.Caption:=IntToStr(Spalten.Value*Zeilen.Value);
end;

procedure TMainForm.FormCreate(Sender: TObject);
begin
  LoadSettings('ImageWalker');
end;

procedure TMainForm.FormDestroy(Sender: TObject);
begin
  SaveSettings('ImageWalker');
end;

procedure TMainForm.StartClick(Sender: TObject);
var
  i,w,h:integer;
  AktImage,ThumbName:string;
begin
  if MakeThumbs.Checked and (ThumbDir.Text='') and (ThumbPrefix.Text='') then
    begin
    MessageDlg('Die Vorschaubilder knnen nicht erzeugt werden, da sie die Orginal-Bilder berschreiben wrden!'+
      'Bitte mindestens Verzeichnis oder Namensprefix fr die Bilder angeben.',
      mtError,[mbOK],0);
    Exit;
    end;

  Cursor:=crHourglass;
  Start.Enabled:=false;
  OptionsGroup.Enabled:=false;
  Verzeichnis.Enabled:=false;
  Laufwerk.Enabled:=true;

  Progress.Max:=Dateien.Items.Count;

  // Create global items in HTML file
  OpenHTMLTemplate(TemplateName,FullDir(Verzeichnis.Directory)+HTMLFile.Text,Dateien.Items.Count,Spalten.Value,Zeilen.Value);
  SetHTMLToken('DATE',DateTimeToStr(Now));
  SetHTMLToken('NUMFILES',IntToStr(Dateien.Items.Count));
  SetHTMLToken('DIRECTORY',Verzeichnis.Directory);

  // Create directory for thumbnails
  if MakeThumbs.Checked and (ThumbDir.Text<>'') then
    begin
    {$I-}
    MkDir(FullDir(Verzeichnis.Directory)+ThumbDir.Text);
    {$I+}
    case IOResult of
      183: Status.SimpleText:='Verzeichnis '+ThumbDir.Text+' existiert bereits.' ;
      0: Status.SimpleText:='Verzeichnis '+ThumbDir.Text+' erzeugt!' ;
      else
        begin
        Status.SimpleText:='Verzeichnis '+ThumbDir.Text+' konnte nicht angelegt werden!!!';
        Exit;
        end;
      end;
    end;

  StartTemplateLoop;
  // Now process all images
  for i := 0 to (Dateien.Items.Count - 1) do
    begin
    Application.ProcessMessages;
    Dateien.ItemIndex:=i;
    Status.SimpleText:='Bearbeitete Datei '+IntToStr(i+1)+' von '+IntToStr(Dateien.Items.Count)+'...';

    AktImage:=FullDir(Verzeichnis.Directory)+Dateien.Items[i];
    InitImage;
    LoadImage(AktImage);
    PreviewImage.Picture.Assign(MainImage);
    PreviewImage.Refresh;

    GetImageSize(w,h);
    SetHTMLToken('PIC_WIDTH',IntToStr(w));
    SetHTMLToken('PIC_HEIGHT',IntToStr(h));
    SetHTMLToken('PIC_URL',Dateien.Items[i]);
    SetHTMLToken('PIC_NAME',Copy(Dateien.Items[i],1,Length(Dateien.Items[i])-Length(ExtractFileExt(Dateien.Items[i]))));
    SetHTMLToken('PIC_SIZE',IntToStr(GetFileSize(AktImage)));

    // Convert the BMPs
    if ConvertBMP.Checked and (LowerCase(ExtractFileExt(AktImage))='.bmp') then
      begin
      AktImage:=ChangeFileExt(AktImage,'.jpg');
      SaveImage(AktImage);
      if DeleteBMP.Checked then
        DeleteFile(FullDir(Verzeichnis.Directory)+Dateien.Items[i]);
      end;

    // Make thumbnails
    if MakeThumbs.Checked or ThumbsPresent.Checked then
      begin
      ThumbName:=FullDir(ExtractFileDir(AktImage)+'\'+ThumbDir.Text)+ThumbPrefix.Text+ExtractFileName(AktImage);

      if MakeThumbs.Checked then
        begin
        while (w>StrToInt(MaxThumbSize.Text)) or (h>StrToInt(MaxThumbSize.Text)) do
          begin
          w:=trunc(w/sqrt(2.0));
          h:=trunc(h/sqrt(2.0));
          end;
        FitToSizeImage(w,h);
        SaveImage(ThumbName);
        end
      else
        if FileExists(ThumbName) then
          begin
          LoadImage(ThumbName);
          GetImageSize(w,h);
          end
        else
          begin
          w:=100;
          h:=100;
          end;

      SetHTMLToken('THUMB_URL',Iif(ThumbDir.Text='','',ThumbDir.Text+'/')+
        ThumbPrefix.Text+ExtractFileName(AktImage));
      SetHTMLToken('THUMB_WIDTH',IntToStr(w));
      SetHTMLToken('THUMB_HEIGHT',IntToStr(h));
      end
    else
      begin
      SetHTMLToken('THUMB_URL','nothing.gif');
      SetHTMLToken('THUMB_WIDTH','100');
      SetHTMLToken('THUMB_HEIGHT','100');
      end;

    FreeImage;

    NextTemplateLoop;
    Progress.Position:=i+1;
    end;

  CloseHTMLTemplate;
  Status.SimpleText:='Fertig!';

  Laufwerk.Enabled:=true;
  Verzeichnis.Enabled:=true;
  OptionsGroup.Enabled:=true;
  Start.Enabled:=true;
  Cursor:=crDefault;
end;

procedure TMainForm.ThumbsPresentClick(Sender: TObject);
begin
  if ThumbsPresent.Checked then
    MakeThumbs.Checked:=false;
  CheckBoxes;
end;

end.
